@@ -8,7 +8,7 @@
 //
 // dType.h: Basic template classes for elementary types
 //
-// Copyright (C) 2013	Xiuwen Zheng
+// Copyright (C) 2007 - 2014	Xiuwen Zheng
 //
 // This file is part of CoreArray.
 //
@@ -29,8 +29,8 @@
  *	\file     dType.h
  *	\author   Xiuwen Zheng
  *	\version  1.0
- *	\date     2007 - 2013
- *	\brief    Basic template classes for elementary types
+ *	\date     2007 - 2014
+ *	\brief    Basic template classes for atomic types
  *	\details
 **/
 
@@ -61,6 +61,117 @@
 
 namespace CoreArray
 {
+	// ******************************************************************** //
+	// ******************************************************************** //
+
+	// define little-endian atomic type
+
+	/// little-endian int8_t
+	#define COREARRAY_LE_INT8_TRAIT_ID            0
+	/// little-endian uint8_t
+	#define COREARRAY_LE_UINT8_TRAIT_ID           1
+	/// little-endian int16_t
+	#define COREARRAY_LE_INT16_TRAIT_ID           2
+	/// little-endian uint16_t
+	#define COREARRAY_LE_UINT16_TRAIT_ID          3
+	/// little-endian int32_t
+	#define COREARRAY_LE_INT32_TRAIT_ID           4
+	/// little-endian uint32_t
+	#define COREARRAY_LE_UINT32_TRAIT_ID          5
+	/// little-endian int64_t
+	#define COREARRAY_LE_INT64_TRAIT_ID           6
+	/// little-endian uint64_t
+	#define COREARRAY_LE_UINT64_TRAIT_ID          7
+
+	/// IEEE 32-bit floating point
+	#define COREARRAY_IEEE_LE_FLOAT32_TRAIT_ID    10
+	/// IEEE 64-bit floating point
+	#define COREARRAY_IEEE_LE_FLOAT64_TRAIT_ID    11
+
+
+	// define big-endian atomic type
+
+	/// big-endian int8_t
+	#define COREARRAY_BE_INT8_TRAIT_ID            20
+	/// big-endian uint8_t
+	#define COREARRAY_BE_UINT8_TRAIT_ID           21
+	/// big-endian int16_t
+	#define COREARRAY_BE_INT16_TRAIT_ID           22
+	/// big-endian uint16_t
+	#define COREARRAY_BE_UINT16_TRAIT_ID          23
+	/// big-endian int32_t
+	#define COREARRAY_BE_INT32_TRAIT_ID           24
+	/// big-endian uint32_t
+	#define COREARRAY_BE_UINT32_TRAIT_ID          25
+	/// big-endian int64_t
+	#define COREARRAY_BE_INT64_TRAIT_ID           26
+	/// big-endian uint64_t
+	#define COREARRAY_BE_UINT64_TRAIT_ID          27
+
+	/// IEEE 32-bit floating point
+	#define COREARRAY_IEEE_BE_FLOAT32_TRAIT_ID    30
+	/// IEEE 64-bit floating point
+	#define COREARRAY_IEEE_BE_FLOAT64_TRAIT_ID    31
+
+
+
+	// define native atomic type
+
+	#if defined(COREARRAY_LITTLE_ENDIAN)
+
+		/// native int8_t
+		#define COREARRAY_NATIVE_INT8_TRAIT_ID      COREARRAY_LE_INT8_TRAIT_ID
+		/// native uint8_t
+		#define COREARRAY_NATIVE_UINT8_TRAIT_ID     COREARRAY_LE_UINT8_TRAIT_ID
+		/// native int16_t
+		#define COREARRAY_NATIVE_INT16_TRAIT_ID     COREARRAY_LE_INT16_TRAIT_ID
+		/// native uint16_t
+		#define COREARRAY_NATIVE_UINT16_TRAIT_ID    COREARRAY_LE_UINT16_TRAIT_ID
+		/// native int32_t
+		#define COREARRAY_NATIVE_INT32_TRAIT_ID     COREARRAY_LE_INT32_TRAIT_ID
+		/// native uint32_t
+		#define COREARRAY_NATIVE_UINT32_TRAIT_ID    COREARRAY_LE_UINT32_TRAIT_ID
+		/// native int64_t
+		#define COREARRAY_NATIVE_INT64_TRAIT_ID     COREARRAY_LE_INT64_TRAIT_ID
+		/// native uint64_t
+		#define COREARRAY_NATIVE_UINT64_TRAIT_ID    COREARRAY_LE_UINT64_TRAIT_ID
+
+		/// native 32-bit floating point
+		#define COREARRAY_IEEE_NATIVE_FLOAT32_TRAIT_ID    COREARRAY_IEEE_LE_FLOAT32_TRAIT_ID
+		/// native 64-bit floating point
+		#define COREARRAY_IEEE_NATIVE_FLOAT64_TRAIT_ID    COREARRAY_IEEE_LE_FLOAT64_TRAIT_ID
+
+	#elif defined(COREARRAY_BIG_ENDIAN)
+
+		/// native int8_t
+		#define COREARRAY_NATIVE_INT8_TRAIT_ID      COREARRAY_BE_INT8_TRAIT_ID
+		/// native uint8_t
+		#define COREARRAY_NATIVE_UINT8_TRAIT_ID     COREARRAY_BE_UINT8_TRAIT_ID
+		/// native int16_t
+		#define COREARRAY_NATIVE_INT16_TRAIT_ID     COREARRAY_BE_INT16_TRAIT_ID
+		/// native uint16_t
+		#define COREARRAY_NATIVE_UINT16_TRAIT_ID    COREARRAY_BE_UINT16_TRAIT_ID
+		/// native int32_t
+		#define COREARRAY_NATIVE_INT32_TRAIT_ID     COREARRAY_BE_INT32_TRAIT_ID
+		/// native uint32_t
+		#define COREARRAY_NATIVE_UINT32_TRAIT_ID    COREARRAY_BE_UINT32_TRAIT_ID
+		/// native int64_t
+		#define COREARRAY_NATIVE_INT64_TRAIT_ID     COREARRAY_BE_INT64_TRAIT_ID
+		/// native uint64_t
+		#define COREARRAY_NATIVE_UINT64_TRAIT_ID    COREARRAY_BE_UINT64_TRAIT_ID
+
+		/// native 32-bit floating point
+		#define COREARRAY_IEEE_NATIVE_FLOAT32_TRAIT_ID    COREARRAY_IEEE_BE_FLOAT32_TRAIT_ID
+		/// native 64-bit floating point
+		#define COREARRAY_IEEE_NATIVE_FLOAT64_TRAIT_ID    COREARRAY_IEEE_BE_FLOAT64_TRAIT_ID
+
+	#else
+	#  error "Unknown endianness"
+    #endif
+
+
+
+
 	/// Memory data type id
 	enum TSVType {
 		svCustom = 0,   ///< Unknown or customized type
@@ -109,42 +220,66 @@ namespace CoreArray
 	#define COREARRAY_TR_INTEGER    1
 	#define COREARRAY_TR_FLOAT      2
 	#define COREARRAY_TR_STRING     3
-	#define COREARRAY_TR_FIXEDSTR   4
 
 
+
+	// ******************************************************************** //
+	// ******************************************************************** //
+
 	// Integers
 
 	/// Signed integer of 8 bits
 	typedef int8_t      Int8;
+	typedef Int8*       PInt8;		
+
 	/// Unsigned integer of 8 bits
 	typedef uint8_t     UInt8;
+	typedef UInt8*      PUInt8;
+
 	/// Signed integer of 16 bits
 	typedef int16_t     Int16;
+	typedef Int16*      PInt16;
+
 	/// Unsigned integer of 16 bits
 	typedef uint16_t    UInt16;
+	typedef UInt16*     PUInt16;
+
 	/// Signed integer of 32 bits
 	typedef int32_t     Int32;
+	typedef Int32*      PInt32;
+
 	/// Unsigned integer of 32 bits
 	typedef uint32_t    UInt32;
+	typedef UInt32*     PUInt32;
+
 	/// Signed integer of 64 bits
 	typedef int64_t     Int64;
+	typedef Int64*      PInt64;
+
 	/// Unsigned integer of 64 bits
 	typedef uint64_t    UInt64;
+	typedef UInt64*     PUInt64;
 
 
 	/// CoreArray Boolean
 	typedef int8_t      CBOOL;
+	typedef CBOOL*      PCBOOL;
 
 
 	#if defined(COREARRAY_MSC) && !defined(ssize_t)
 	typedef ptrdiff_t	ssize_t;
 	#endif
 
+
+	// ******************************************************************** //
+	// ******************************************************************** //
+
 	// Integer Traits
 
 	template<typename T> struct TdTraits
 	{
     	typedef T TType;
+		typedef T ElmType;
 		static const int trVal = COREARRAY_TR_UNKNOWN;
 		static const unsigned BitOf = sizeof(T)*8u;
 		static const bool isClass = false;
@@ -154,6 +289,7 @@ namespace CoreArray
 	template<> struct TdTraits<Int8>
 	{
 		typedef Int8 TType;
+		typedef Int8 ElmType;
 		static const int trVal = COREARRAY_TR_INTEGER;
 		static const unsigned BitOf = 8u;
 		static const bool isClass = false;
@@ -169,6 +305,7 @@ namespace CoreArray
 	template<> struct TdTraits<UInt8>
 	{
 		typedef UInt8 TType;
+		typedef UInt8 ElmType;
 		static const int trVal = COREARRAY_TR_INTEGER;
 		static const unsigned BitOf = 8u;
 		static const bool isClass = false;
@@ -186,6 +323,7 @@ namespace CoreArray
 	template<> struct TdTraits<Int16>
 	{
 		typedef Int16 TType;
+		typedef Int16 ElmType;
 		static const int trVal = COREARRAY_TR_INTEGER;
 		static const unsigned BitOf = 16u;
 		static const bool isClass = false;
@@ -201,6 +339,7 @@ namespace CoreArray
 	template<> struct TdTraits<UInt16>
 	{
 		typedef UInt16 TType;
+		typedef UInt16 ElmType;
 		static const int trVal = COREARRAY_TR_INTEGER;
 		static const unsigned BitOf = 16u;
 		static const bool isClass = false;
@@ -216,6 +355,7 @@ namespace CoreArray
 	template<> struct TdTraits<Int32>
 	{
 		typedef Int32 TType;
+		typedef Int32 ElmType;
 		static const int trVal = COREARRAY_TR_INTEGER;
 		static const unsigned BitOf = 32u;
 		static const bool isClass = false;
@@ -231,6 +371,7 @@ namespace CoreArray
 	template<> struct TdTraits<UInt32>
 	{
 		typedef UInt32 TType;
+		typedef UInt32 ElmType;
 		static const int trVal = COREARRAY_TR_INTEGER;
 		static const unsigned BitOf = 32u;
 		static const bool isClass = false;
@@ -246,6 +387,7 @@ namespace CoreArray
 	template<> struct TdTraits<Int64>
 	{
 		typedef Int64 TType;
+		typedef Int64 ElmType;
 		static const int trVal = COREARRAY_TR_INTEGER;
 		static const unsigned BitOf = 64u;
 		static const bool isClass = false;
@@ -261,6 +403,7 @@ namespace CoreArray
 	template<> struct TdTraits<UInt64>
 	{
 		typedef UInt64 TType;
+		typedef UInt64 ElmType;
 		static const int trVal = COREARRAY_TR_INTEGER;
 		static const unsigned BitOf = 64u;
 		static const bool isClass = false;
@@ -277,7 +420,7 @@ namespace CoreArray
 
 	// Bit Type
 
-	namespace Internal
+	namespace _Internal_
 	{
 		template<unsigned bits> struct _BitIndex
 		{
@@ -343,15 +486,17 @@ namespace CoreArray
 		static const unsigned NumBit = (bits > 0) ? bits : (-bits);
 		static const bool Sign = (bits > 0);
 		static const unsigned Flag = 1u << (NumBit-1);
+
 	private:
-		static const int _index = Internal::_BitIndex<NumBit>::Val;
-		typedef typename Internal::_Bit2Int<_index, true>::Type _IntType;
-		typedef typename Internal::_Bit2Int<_index, true>::TypeEx _IntTypeEx;
-		static const _IntType _IntVal = Internal::_Bit2Int<_index, true>::Val;
-		static const _IntTypeEx _IntValEx = Internal::_Bit2Int<_index, true>::ValEx;
+		static const int _index = _Internal_::_BitIndex<NumBit>::Val;
+		typedef typename _Internal_::_Bit2Int<_index, true>::Type _IntType;
+		typedef typename _Internal_::_Bit2Int<_index, true>::TypeEx _IntTypeEx;
+		static const _IntType _IntVal = _Internal_::_Bit2Int<_index, true>::Val;
+		static const _IntTypeEx _IntValEx = _Internal_::_Bit2Int<_index, true>::ValEx;
+
 	public:
-		typedef typename Internal::_Bit2Int< _index, bits<0 >::Type IntType;
-		typedef typename Internal::_Bit2Int< _index, bits<0 >::TypeEx IntTypeEx;
+		typedef typename _Internal_::_Bit2Int< _index, bits<0 >::Type IntType;
+		typedef typename _Internal_::_Bit2Int< _index, bits<0 >::TypeEx IntTypeEx;
 
 		static const IntType Mask = ~(_IntVal << NumBit);
 		static const _IntType NOTMask = (_IntVal << NumBit);
@@ -413,6 +558,7 @@ namespace CoreArray
 	template<int bits> struct TdTraits< BITS<bits> >
 	{
 		typedef typename BITS<bits>::IntType TType;
+		typedef typename BITS<bits>::IntType ElmType;
 		static const int trVal = COREARRAY_TR_INTEGER;
 		static const unsigned BitOf = (bits > 0) ? bits : (-bits);
 		static const bool isClass = false;
@@ -446,11 +592,12 @@ namespace CoreArray
 	typedef long double LongFloat;
 
 
-	// Float Traits
+  	// Float Traits
 
 	template<> struct TdTraits<Float32>
 	{
 		typedef Float32 TType;
+		typedef Float32 ElmType;
 		static const int trVal = COREARRAY_TR_FLOAT;
 		static const unsigned BitOf = 32u;
 		static const bool isClass = false;
@@ -468,6 +615,7 @@ namespace CoreArray
 	template<> struct TdTraits<Float64>
 	{
 		typedef Float64 TType;
+		typedef Float64 ElmType;
 		static const int trVal = COREARRAY_TR_FLOAT;
 		static const unsigned BitOf = 64u;
 		static const bool isClass = false;
@@ -485,6 +633,7 @@ namespace CoreArray
 	template<> struct TdTraits<LongFloat>
 	{
 		typedef LongFloat TType;
+		typedef LongFloat ElmType;
 		static const int trVal = COREARRAY_TR_FLOAT;
 		static const unsigned BitOf = sizeof(LongFloat)*8u;
 		static const bool isClass = false;
@@ -534,11 +683,11 @@ namespace CoreArray
 #endif
 
 	/// UTF-8 string
-	typedef std::string					UTF8String;
+	typedef std::basic_string<UTF8>     UTF8String;
 	/// UTF-16 string
-	typedef std::basic_string<UTF16>	UTF16String;
+	typedef std::basic_string<UTF16>    UTF16String;
 	/// UTF-32 string
-	typedef std::basic_string<UTF32>	UTF32String;
+	typedef std::basic_string<UTF32>    UTF32String;
 
 
 	// String Traits
@@ -552,20 +701,19 @@ namespace CoreArray
 		static const bool isClass = true;
 		static const TSVType SVType = svStrUTF8;
 
-		static const char * TraitName() { return "UTF8"; }
+		static const char * TraitName() { return "Str8"; }
 	};
 
 	template<> struct TdTraits<UTF8*>
 	{
 		typedef UTF8String TType;
 		typedef UTF8 ElmType;
-		static const int trVal = COREARRAY_TR_FIXEDSTR;
+		static const int trVal = COREARRAY_TR_STRING;
 		static const unsigned BitOf = 8u;
 		static const bool isClass = false;
 		static const TSVType SVType = svStrUTF8;
 
-		static const char * TraitName() { return "FStr8"; }
-		static const char * StreamName() { return "dFStr8"; }
+		static const char * TraitName() { return "Str8"; }
 	};
 
 	template<> struct TdTraits<UTF16String>
@@ -577,20 +725,19 @@ namespace CoreArray
 		static const bool isClass = true;
 		static const TSVType SVType = svStrUTF16;
 
-		static const char * TraitName() { return "UTF16"; }
+		static const char * TraitName() { return "Str16"; }
 	};
 
 	template<> struct TdTraits<UTF16*>
 	{
 		typedef UTF16String TType;
 		typedef UTF16 ElmType;
-		static const int trVal = COREARRAY_TR_FIXEDSTR;
+		static const int trVal = COREARRAY_TR_STRING;
 		static const unsigned BitOf = 16u;
 		static const bool isClass = false;
 		static const TSVType SVType = svStrUTF16;
 
-		static const char * TraitName() { return "FStr16"; }
-		static const char * StreamName() { return "dFStr16"; }
+		static const char * TraitName() { return "Str16"; }
 	};
 
 	template<> struct TdTraits<UTF32String>
@@ -602,24 +749,23 @@ namespace CoreArray
 		static const bool isClass = true;
 		static const TSVType SVType = svCustomStr;
 
-		static const char * TraitName() { return "UTF32"; }
+		static const char * TraitName() { return "Str32"; }
 	};
 
 	template<> struct TdTraits<UTF32*>
 	{
 		typedef UTF32String TType;
 		typedef UTF32 ElmType;
-		static const int trVal = COREARRAY_TR_FIXEDSTR;
+		static const int trVal = COREARRAY_TR_STRING;
 		static const unsigned BitOf = 32u;
 		static const bool isClass = false;
 		static const TSVType SVType = svCustomStr;
 
-		static const char * TraitName() { return "FStr32"; }
-		static const char * StreamName() { return "dFStr32"; }
+		static const char * TraitName() { return "Str32"; }
 	};
 
 
-	/// Customized type
+	/// Customized integer type
 	/** \tparam TYPE  any data type, e.g integer or float number
 	 *  \tparam SIZE  to specify the structure size, can be != sizeof(TYPE)
 	**/
@@ -656,6 +802,28 @@ namespace CoreArray
 	private:
 		TYPE fVal;
     };
+
+
+	/// fixed-length array
+	/** \tparam TYPE  any data type, e.g integer or float number
+	 *  \tparam SIZE  the array length
+	**/
+	template<typename TYPE, int SIZE=0> struct FIXED_LENGTH
+	{
+		typedef typename TdTraits<TYPE>::TType TType;
+		typedef typename TdTraits<TYPE>::ElmType ElmType;
+		TYPE Value[SIZE];
+	};
+
+	/// variable-length array
+	/** \tparam TYPE  any data type, e.g integer or float number
+	**/
+	template<typename TYPE> struct VARIABLE_LENGTH
+	{
+		typedef typename TdTraits<TYPE>::TType TType;
+		typedef typename TdTraits<TYPE>::ElmType ElmType;
+	};
+
 }
 
 #endif /* _dType_H_ */
